/**
 * @file
 *
 * @copyright Axiros GmbH 2014-2022
 *
 * @brief Configuration parameters and config options for AXACT fine tuning
 */

#ifndef CCONFIG_H_
#define CCONFIG_H_

/*! \page cconfig Global configuration parameters
These options allow the fine tuning of AXACT to fit into the target system

The options are grouped into:
\li \ref gccfile
\li \ref gccnet
\li \ref gcccwmp
\li \ref gccssl
\li \ref gccfeat

*/


/***** BEGIN GLOBALS *****/

/****************************************************/
/** @defgroup gccfile Directory and Filenames
 * Definition of Sysconfig directory and filenames
 * @{ ***********************************************/

/** name of the database file (read/write access required) */
#define AXACT_DB_FILE "db.xml"

/** name of the logfile (write access required) */
#define AXACT_LOGFILE "axact.log"

/** Size limit (in KB) per general logfile. Two can be generated and the oldest exceeding
  * (2 * AXACT_LOGFILE_MAX_SIZE) will be discarded.
  * 0 disable rotation.
  * Only used in daemon mode (write access required) */
#define AXACT_LOGFILE_MAX_SIZE 100

/** name of the bootstrap file (read/write access required) */
#define CWMP_BOOTSTRAP_FILE ".bootstrap"

/** name of the serialised TR-069 events file (read/write access required) */
#define CWMP_STATUS_FILE "status"

/** name of the serialised TR-069 metadata file (read/write access required) */
#define CWMP_METADATA_FILE "metadata"

/** comma separated list of log methods to use by default (if supported)
 *  currently supported:
 *  - console
 *  - file
 *  - syslog
 *
 *  if this is left empty then no log will be output after initialisation
 */
#define DEFAULT_LOG_TYPE "console"

/** @} */

/****************************************************/
/** @defgroup gccnet Network and protocol settings
 * Definition of Network and protocol settings
 * @{ ***********************************************/

/** timeout for network connection establishment */
#define CONNECTION_TIMEOUT 10


/** timeout for established ACS connections */
#define ACS_SESSION_TIMEOUT 30


/** Set User Agent for HTTP messages */
#define HTTP_USER_AGENT "AXACT/" PACKAGE_VERSION


/** local HTTP connection request port */
#define HTTP_CR_PORT 8090


/** Interval between automatic internal IP change checks */
#define HTTP_CR_UPDATE_INTERVAL 600


/** set to 0 to always authenticate connection requests, 1 is default */
#define AUTHENTICATE_CONNECTION_REQUEST 1


/** Maximum number of authentication attempts */
#define HTTP_MAX_AUTHENTICATION 3

/** @} */


/****************************************************/
/** @defgroup gcccwmp CWMP protocol settings
 * Definition of CWMP protocol (TR-069) related settings
 * @{ ***********************************************/

/** Send UTF-8 to the ACS or use XML NCR instead? */
#define CWMP_USE_UTF8 1

/** @} */




/****************************************************/
/** @defgroup gccssl SSL/TLS encryption
 * Definition of SSL/TLS encryption related settings
 * @{ ***********************************************/

/****************************************************/
/** @defgroup gccassl Server side certificate support
 * Definition of the server side SSL/TLS encryption related settings
 * @{ ***********************************************/
/** Strict means we're only going to allow TLS 1.2 and later and no SSL
 *
 * TR-069 Amendment 5 requires the use of TLS v1.2 (or later) for communication.
 * @note For TLS v1.2 support the minimum needed OpenSSL version on both sides is 1.0.1.
 * @note Due to the general availability of TLS 1.2 everywhere the default has
 *       been changed starting with version 3.2.7 and this configuration is
 *       deprecated now
 */
#define TLS_STRICT_MODE 1

/** Disable SSLv2 and SSLv3 protocols?
 *
 * SSLv2 and SSLv3 are insecure beyond repair, cf.
 * http://en.wikipedia.org/wiki/POODLE. The only reliable workaround is
 * disabling of SSLv2 and SSLv3 protocols by preventing their negotiation with
 * the server.
 *
 * This setting should only be changed in the unlikely event that the server
 * does not support TLS1.0 or higher which unlikely.
 */
#define TLS_DISABLE_SSLv2v3 1

/** The default path to the CA cert bundle for CA chain verification 
 *
 * This defines the full default filename for the CA certificate or bundle in PEM format. 
 * If CA chain verification is active the client will load the bundle and use it for verification 
 * of the presented certificate.
 *
 * By default this path is dependent on the chosen sysconfdir (which usually depends on the chosen prefix) 
 * but it can also be a fixed path if needed
 */
#define TLS_CA_BUNDLE_FILE "cacerts.pem"

/** Do we want to verify the validity of the chained signatures of the presented server certificate
 *  against the specified CA bundle? 
 *
 *  If activated and the verification fails the communication is aborted.
 */
#define TLS_VERIFY_CA_CHAIN 1

/** Do we want to verify that the one of the Subject Alternative Names DNS
 * entries matches the ACS hostname?
 *
 * Please note that TR-069a4 always requires checking of the CN while setting
 * this option can potentially bypass the CN check. But since TR-069a4 is way
 * outdated and TR-069a6 recommends checking the SANs as well this will in
 * version 2022.3 and later default to on.
 */
#define TLS_VERIFY_SAN 1

/** Do we want to verify that the host name matches the CN in the cert? 
 *
 * This enables the verifcation of the common name in the presented server certificate against 
 * the hostname/IP used in the ACS URL. The verification also supports wildcard certificates.
 * 
 * However if a hostname is specified in the certificates only the hostname may be used in the ACS URL
 * and vice versa for IP addresses, no attempt to resolve names or ip addresses is made as this is expressly 
 * forbidden by TR-069 Amendment 4.
 */
#define TLS_VERIFY_CN 1

/** Verify that the certificate is currently valid?
 *
 * Verifies the notValidBefore and notValidAfter fields inside the presented server certificate
 * against the current time. It is necessary to have a valid time on the CPE for the validation to work.
 */
#define TLS_VERIFY_VALIDITY 1

/** @} */
/****************************************************/
/** @defgroup gcccssl Client side certificate support
 * Definition of the client side SSL/TLS encryption related settings
 * @{ ***********************************************/

/** Do we want to enforce the use of client certificates?
 *
 * If activated, AXACT will enforce the use of client certificates, i.e. it can
 * not be operated without.  Otherwise AXACT will look for the existence of the
 * certificate and use it if available.
 */
#define TLS_CLIENT_USE_CERTIFICATES 0

/** The default path to the client certificate
 *
 * This can also be a combined file containing a certificate and the private
 * key in which case TLS_CLIENT_KEY_FILE is not going to be used
 */
#define TLS_CLIENT_CERT_FILE "client.pem"

/** The default path to the client key 
 *
 * If set this will specify the location of the key to access the certificate
 * that is sent to the server.
 */
#define TLS_CLIENT_KEY_FILE "clientkey.pem"

/** @} */
/** @} */


 /****************************************************/
/** @defgroup gccfeat Additional Features
 * Definition of Additional Feature configuration settings
 * in file cconfig.h.in
 * @{ ***********************************************/

/** Use XMPP Ping feature for keepalive connection testing */
#define XMPP_USE_PING 1

/** Enforce use of STARTTLS even if server doesn't advertise it (RFC 7590) */
#define XMPP_ENFORCE_STARTTLS 1

/** DBus: Use session (DBUS_BUS_SESSION) or system (DBUS_BUS_SYSTEM) bus? */
#define AXACT_DBUS_BUS_TYPE DBUS_BUS_SYSTEM

/** DBus: Interface name to register on the bus */
#define AXACT_DBUS_SERVICE_NAME "com.axiros.axact.AxactService"

/** DBus: Object name to use for client internals */
#define AXACT_DBUS_OBJECT_PATH "/com/axiros/axact/AxactService"

/** @} */ 


#endif /* CCONFIG_H_ */
