/**
 * @file
 *
 * @copyright Axiros GmbH 2010-2022, All rights reserved
 *
 * @brief Functions for diagnostic and operational status output definitions
 */

#ifndef LOGGER_H_
#define LOGGER_H_

#include "compiler.h"


/*! \page logging Logging
The logging function allows to log messages either to SYSLOG or a file depending on the overall configuration.
For a general description about the logging configuration see the product documentation.

<hr>
\section slog2 Debug logging

To enable or disable debug log output the compiler flag "DEBUGGING" is used, which is controlled over the configure
option --enable-debug. To prevent debug logging calls from showing up if no debug logging is enabled the call to
the logging function has to be wrapped into the compiler option:
\code
#ifdef DEBUGGING
    logg(DEBUG, ... );
#endif
\endcode

For convenience the debug macro LDEBUG is provided, which should be used instead of the
\#ifdef DEBUGGING encapsulation.

<hr>
All definitions are in the \ref logger.h file.

Back to \ref interfaces "Interfaces".

*/

/** @defgroup glog Logging function
 * The logging function allows to log messages either to SYSLOG or a file depending on the overall configuration.
 * To use logging in a source file it needs to include <b>"core/logger.h"</b>.
 * @{
 */

/** Loglevels used by AXACT */
enum loglevel {DEBUG, INFO, WARNING, ERR, CRITICAL, ANY};


#ifdef DEBUGGING
	#define LDEBUG(...)  logg(DEBUG, __VA_ARGS__)
#else
	#define LDEBUG(...)
#endif


/**
 * @brief Logging function of AXACT
 *
 * @param[in] loglevel logging priority, must be one of the values defined in loglevel
 * @param[in] format output as format string (with variable number of arguments following) or output string
 *
 * @note For Debug logging use the provided debug macros LDEBUG, which is guarded by the DEBUGGING compile time flag.
 */
void logg(int loglevel, const char* format, ...) ARGS_NONNULL ARGS_FORMAT_PRINTF(2,3);

/**@}*/


/**
 * @brief Initialize the logger and open streams.
 */
void logger_init (void) DEPRECATED("Logging is always available, to configure it use logger_configure_channels");


/**
 * @brief Configure the output channel
 *
 * @param[in] channels	the channels to use output log information, takes a comma separated list of console, file and/or syslog
 */
void logger_configure_channels (char *channels) ARGS_NONNULL;


/**
 * @brief Configure the name of the logfile to use
 *
 * @param[in] name	a filename path to the logfile to append to, can be NULL to disable file logging
 */
void logger_configure_logfile (char *name);


/**
 * @brief Finish the logger and clean resources.
 */
void logger_cleanup(void) DEPRECATED("It's a no-op now");


/**
 * @brief Set new loglevel to be used
 *
 * @param[in] loglevel logging priority, must be one of the defined values
 */
void set_loglevel(int loglevel);


#endif /* LOGGER_H_ */
