/**
 * @file
 *
 * @copyright Axiros GmbH 2022, All rights reserved
 *
 * @brief Object timer acces and handling functions
 */

#ifndef OBJECTTIMER_H_
#define OBJECTTIMER_H_

#include <time.h>

#include "compiler.h"

struct object_timer;
struct dm_object;
struct frontend;

/* Definition callback function to be called on object timer timeout */
typedef void (*object_timer_cb) (struct frontend *fe, char *path, struct object_timer *obj_timer, void *arg);


/**
 * @brief Create and start a one-shoot timer linked to a particular object
 *        Periodic functionality can be achieved by reseting the timer in the callback on expiration
 *        Resources are automatically freed after the timeout
 *
 * @param[in] obj		object to link the timer to
 * @param[in] timeout	timeout in seconds
 * @param[in] cb		callback function executed on expiration
 * @param[in] arg		passed to callback function
 *
 * @return 0 if successful, 1 otherwise
 */
int object_timer_create (struct dm_object *obj, time_t timeout, object_timer_cb cb, void *arg) ARGINDEX_NONNULL(1) ARGINDEX_NONNULL(3);


/**
 * @brief Restart previously created object timer with a new timeout
 *
 * @param[in] obj_timer object timer
 * @param[in] timeout timeout in seconds
 *
 * * @return 0 if successful, 1 otherwise
 */
int object_timer_reset (struct object_timer *obj_timer, time_t timeout) ARGINDEX_NONNULL(1);


/**
 * @brief Stop and free the object timer, while maintaining the list.
 *        If the obj_timer points to the list head node, the head pointer is set to NULL.
 *
 * @param[in] obj_timer
 *
 */
void object_timer_delete (struct object_timer *obj_timer) ARGS_NONNULL;


/**
 * @brief Get the first object timer of a given frontend
 *
 * @param[in] fe
 *
 * @return pointer to the head list node or NULL
 */
struct object_timer *object_timer_get_first (struct frontend *fe) ARGS_NONNULL;


/**
 * @brief Get the next object timer in the list
 *
 * @param[in] obj_timer
 *
 * @return pointer to the next object timer in the list or NULL
 */
struct object_timer *object_timer_get_next (struct object_timer *obj_timer) ARGS_NONNULL;


/**
 * @brief Get the object timer attached to a specific datamodel object
 *
 * @param[in] obj 	a pointer to the datamodel object
 *
 * @return pointer to the matching object timer or NULL
 */
struct object_timer *object_timer_get_for_dm_object (struct dm_object *obj) ARGS_NONNULL;


/**
 * @brief Get the next object timer attached to the same datamodel object as the current object timer
 *
 * @param[in] obj_timer a pointer to the object timer
 *
 * @return pointer to the next matching object timer or NULL
 */
struct object_timer *object_timer_get_next_for_dm_object (struct object_timer *obj_timer) ARGS_NONNULL;
#endif
