/**
 * @file
 *
 * @copyright Axiros GmbH 2014-2022, All rights reserved
 *
 * @brief Definition of data model functions and structures
 */

#include "config.h"

#ifndef DATAMODELDEF_H_
#define DATAMODELDEF_H_

#include "datamodel/DataModel.h"

#if defined (TR069_XMPP_SUPPORT)
#define DMDEF_XMPPCRSUPPORT "HTTP,XMPP"
#else
#define DMDEF_XMPPCRSUPPORT "HTTP"
#endif

#define DMDEF_IPV6SUPPORT "1"

/**
 * @brief Parameter properties used in data model
 */
#define pWRITE		0x0001      /**< writable parameter  */
#define pNOREAD		0x0002      /**< hidden parameter, GPV will return empty value, e.g. passwords  */
#define pACTION		0x0004      /**< action parameter, setting will cause action, GPV will return empty value, e.g. reset */
#define pNONOTIFYCHANGE	0x0008      /**< nonotify: change of notification state is denied */
#define pKEY		0x0020      /**< key: parameter is a table key */
#define pKEYTYPE	0x0040      /**<      type of key (0: single, 1 part) */
#define pREF		0x0080      /**< ref: parameter is a reference */
#define pREFTYPE	0x0100      /**<      type of reference (0: weak, 1 strong) */
#define pNOBACKUP	0x0200      /**< parameter is volatile no value backup is performed */
#define pLIST		0x0400      /**< list parameter */
#define pDEFACTIVE	0x0800      /**< parameter will have active notification enabled by default */
#define pFORCEDINFORM	0x1000      /**< parameter will be always included in every inform */
#define pMANDATORY	0x2000      /**< parameter is mandatory to be used, used for USP command input arguments */


/**
 * @brief Parameter structure used in data model
 */
struct dm_parameter_template {
	/*@{*/
	char*              _name;		/**< reference to parameter name */
	char*              _value;		/**< reference to parameter default value */
	void		  *_syntax_details;	/**< pointer to type specific details */
	dm_syntax_facet	   _syntax_type;	/**< type of the syntax check */
	dm_xsd_type        _type;		/**< parameter type */
	unsigned short     _properties;		/**< parameter properties */
	/*@}*/
};


/**
 * @brief Object properties used in data model (max number 16)
 */
#define oADD		0x0001	/**< Object can be added */
#define oDELETE		0x0002	/**< Object can be deleted */
#define oTABLE		0x0004	/**< Object is a table */
#define oTABLEINSTANCE	0x0008	/**< Object is a table instance */
#define oISREFFERED	0x0010	/**< Object is referenced in data model */
#define oOVERLAY	0x0020	/**< Object contains overlaid parameters */
#define oEVENT		0x0040	/**< Special object representing a USP event */
#define oCOMMAND	0x0080	/**< Special object representing a USP command */
#define oDATA		0x0100	/**< Special object representing arbitrary invisible data */
#define oNOBACKUP	0x0200	/**< Do not backup this or any objects underneath */
#define oASYNC		0x0400	/**< USP command is asynchronous */


/**
 * @brief Object structure used in data model
 *
 * @details This structure is the data model internal representation of a
 *          TR-069 object and/or table instance or an table root.
 *
 * @note If the object is a table root object some parameters are used differently form a standard object
 *          or instance
 */
struct dm_object_template
{
	char				*_name;          /**< Node name */
	dm_readfunc			_read_proc;     /**< Pointer to Read procedure / init procedure for table root object*/
	dm_writefunc			_write_proc;    /**< Pointer to Write procedure / write procedure for table root object*/
	struct dm_parameter_template	*_parameters;    /**< Pointer to parameter array / pointer to template for table root object*/
	short				_properties;
};


#endif /* DATAMODELDEF_H_ */
