/**
 * @file
 *
 * @copyright Axiros GmbH 2012-2022, All rights reserved
 *
 * @brief Definition of functions and structures for the south bound (OS) interface
 */

 /*! \page dmosinterfaces DMOS Interface
The DMOS interface functions form the Mapping Support Library, which provides access to the data model
and allow working with the data model objects and parameters.

The DMOS interface functions are grouped into:
\li \ref dmos_acc DMOS Object Access functions to retrieve objects and parameters
\li \ref dmos_pacc DMOS Parameter Access functions to retrieve parameters or their values
\li \ref dmos_set DMOS Set functions to change values in the data model
\li \ref dmos_prop DMOS Object property routines to access individual object properties
\li \ref dmos_pprop DMOS Parameter properties access routines to access individual parameter properties
\li \ref dmos_table DMOS Table handling routines to work with table instances
\li \ref dmos_misc DMOS miscellaneous routines

\li \ref osdm_out OSDM Out of session functions, which may only be called on asynchronous events

All functions are defined in the \ref dmosmap.h header file.

Back to \ref interfaces "Interfaces".
*/

#ifndef DMOS_H_
#define DMOS_H_

#include <time.h>

#include "compiler.h"


/* Forward declarations */
struct dm_object;
struct dm_parameter;


/**
 * @brief Initialize OS specific part of data model
 *
 * @param[in] root root object of the datamodel
 */
void dmos_init (struct dm_object *root) ARGS_NONNULL DEPRECATED("This function is a no-op since version 2019.3");


/** @defgroup dmos_acc DMOS Object access Routines
 * Object access functions are used to access a specific object form the data model. It can either be addressed
 * by the path name or relative to a known object to allow navigating the object tree.
 * @{
 */

/**
 * @brief Get object referenced by path name
 *
 * @param[in] root root object of the datamodel
 * @param[in] path absolute path name of object
 *
 * @return pointer to struct dm_object *if successful or NULL
 *
 * This is the basic access function for any object in the data model. To
 * retrieve a object from the data model specify the path: e.g.
 * dmos_getObject (dm_root, "Device.IP."). The path may also leave out the root
 * model to operate root model independent, e.g. dmos_getObject ("Device.IP.")
 */
struct dm_object *dmos_getObjectFromRoot (struct dm_object *root, char *path) ARGS_NONNULL;


/**
 * @brief get ManagementServer Object
 *
 * @param[in] obj	pointer to **any** object in the datamodel
 *
 * @return pointer to the Device.ManagementServer. object in this datamodel
 */
struct dm_object *dmos_getManagementServerObject (struct dm_object *obj) ARGS_NONNULL;


/**
 * @brief get DeviceInfo Object
 *
 * @param[in] obj	pointer to **any** object in the datamodel
 *
 * @return pointer to the Device.DeviceInfo. object in this datamodel
 */
struct dm_object *dmos_getDeviceInfoObject (struct dm_object *obj) ARGS_NONNULL;


/**
 * @brief get ManagementServer Object
 *
 * @param[in] obj	pointer to **any** object in the datamodel
 *
 * @return pointer to the Device.LocalAgent. object in this datamodel
 */
struct dm_object *dmos_getLocalAgentObject (struct dm_object *obj) ARGS_NONNULL;


/**
 * @brief get referenced Object
 *
 * @param[in] root	pointer to the root object of the datamodel
 * @param[in] path	TR-106 path reference to Object (Path name without '.' at the end)
 *
 * @return pointer to object maching the given path reference or NULL
 */
struct dm_object *dmos_get_ReferenceObj (struct dm_object *root, char *path) ARGS_NONNULL;


/**
 * @brief get SubObject of the given object
 * @param[in] obj  Pointer to object
 * @param[in] subname Name of the sub object
 *
 * @return pointer to sub object, or NULL if not found
 *
 * @details Only returns first level subs, if a table is used it returns the instance
 *            e.g :  obj points to Device.DeviceInfo., to retrieve Device.DeviceInfo.MemoryStatus.
 * call <b>dmos_getSubObject (obj, "MemoryStatus")</b>
 */
struct dm_object *dmos_getSubObject (struct dm_object *obj, char *subname) ARGS_NONNULL;


/**
 * @brief get a pointer to the parent object of given object
 * @param[in] obj  Pointer to the object
 *
 * @return pointer to parent object, or NULL if not found
 */
struct dm_object *dmos_getParentObject (struct dm_object *obj) ARGS_NONNULL;


/**
 * @brief get a pointer to the root object of the datamodel
 *
 * @param[in] obj	pointer to any object in the datamodel
 *
 * @return pointer to the root object of the datamodel
 */
struct dm_object *dmos_getRootObject (struct dm_object *obj) ARGS_NONNULL RETURNS_NONNULL;


/**@}*/


/* == Set routines  == */
/** @defgroup dmos_pacc DMOS Parameter access Routines
 * Parameter access functions are used to access a specific parameter form the data model. For the generic
 * access functions the parameter is addressed relative to its object object. In addition there are a specialized
 * convenience functions provided.
 * @{
 */

/**
 * @brief find parameter structure in object
 * @param[in] obj  Pointer to object
 * @param[in] parametername Name of parameter
 *
 * @return pointer to parameter structure or NULL if not found
 */
struct dm_parameter *dmos_getObjectParameter (struct dm_object *obj, char *parametername) ARGS_NONNULL;


/**
 * @brief get the value of the given parameter
 * @param[in] obj  Pointer to object
 * @param[in] parametername Name of parameter
 *
 * @return pointer value or NULL if not found
 *
 * @details This function first checks if there is an requested change. If yes, the new
 * value for the parameter is returned, otherwise the value stored in the
 * data model.
 */
char *dmos_getObjectParameterValue (struct dm_object *obj, char *parametername) ARGS_NONNULL;


/**
 * @brief get the pending value of the given parameter
 * @param[in] obj  Pointer to object
 * @param[in] parametername Name of parameter
 *
 * @return pointer value or NULL if no pending value is found
 *
 * @details This function first checks if there is an requested change. If yes, the new
 * value for the parameter is returned, otherwise NULL
 */
char *dmos_getPendingParameterValue (struct dm_object *obj, char *parametername) ARGS_NONNULL;


 /**
 * @brief get the value of an boolean parameter in the object
 * @param[in] obj  Pointer to object
 * @param[in] parametername Name of parameter
 *
 * @return boolean value (0 or 1) or if not found -1
 *
 * @details This function first checks if there is an requested change. If yes, the new
 * value for the parameter is returned, otherwise the value stored in the
 * data model.
 */
int dmos_getBooleanParameterValue (struct dm_object *obj, char *parametername) ARGS_NONNULL;


/**
* @brief Return whether the boolean value of the Enable parameter on the given object is true or doesn't exist
* @param[in] obj  Pointer to object
*
* @return boolean value (0 or 1) or -1 if object doesn't contain a parameter called Enable
*/
int dmos_isEnabled (struct dm_object *obj) ARGS_NONNULL;


 /**
 * @brief get the value of an unsigned integer parameter in the object
 * @param[in] obj  Pointer to object
 * @param[in] parametername Name of parameter
 * @param[out] err error code (0 no error, 1 value is undefined)
 *
 * @return unsigned integer value
 *
 * @details This function first checks if there is an requested change. If yes, the new
 * value for the parameter is returned, otherwise the value stored in the
 * data model.
 */
unsigned int dmos_getUnsignedParameterValue (struct dm_object *obj, char *parametername, int *err) ARGS_NONNULL;

 /**
 * @brief get the value of an integer parameter in the object
 * @param[in] obj  Pointer to object
 * @param[in] parametername Name of parameter
 * @param[out] err error code (0 no error, 1 value is undefined)
 *
 * @return unsigned integer value
 *
 * @details This function first checks if there is an requested change. If yes, the new
 * value for the parameter is returned, otherwise the value stored in the
 * data model.
 */
int dmos_getIntParameterValue (struct dm_object *obj, char *parametername, int *err) ARGS_NONNULL;


/**
 * @brief set lower layer
 * @param[in] obj  stackable object instance
 * @param[in] lowobj stackable lower object instance
 *
 * @return 0 if successful, or error code
 *
 * @details  sets the lower layer parameter and updates the interface stack object
 *           specific for DEVICE:2 data model
 */
unsigned int dmos_setObjectLowerLayer (struct dm_object *obj, struct dm_object *lowobj) ARGS_NONNULL;


/**
 * @brief get Lower Layer Object
 * @param[in] obj  stackable object instance
 *
 * @return pointer to lower layer object , NULL if no lower layer exists
 */
struct dm_object *dmos_getLowerLayerObject (struct dm_object *obj) ARGS_NONNULL;


/**@}*/

/* == Set routines  == */
/** @defgroup dmos_set DMOS Set Routines
 * Routines to change object and parameter values
 * @{
 */

/**
 * @brief Set object parameter to new value
 * @param[in] obj  Pointer to object
 * @param[in] parametername Name of parameter
 * @param[in] value New value as char pointer
 *
 * @return 0 or TR069 error code
 */
unsigned int dmos_setObjectParameter (struct dm_object *obj, char *parametername, char *value) ARGS_NONNULL;


/**
 * @brief Set object parameter to new value no logging
 * @param[in] obj  Pointer to object
 * @param[in] parametername Name of parameter
 * @param[in] value New value as char pointer
 *
 * @details same function like dmos_setObjectParameter, if the parameter is not found no log
 *          entry is created
 *
 * @return 0 or TR069 error code
 */
unsigned int dmos_setObjectParameterNoLog (struct dm_object *obj, char *parametername, char *value) ARGS_NONNULL;


/**
 * @brief Set object parameter to new value
 * @param[in] obj  Pointer to object
 * @param[in] parametername Name of parameter
 * @param[in] value New value as unsigned integer
 *
 * @return 0 or TR069 error code
 */
unsigned int dmos_setObjectParameterUnsignedInt (struct dm_object *obj, char *parametername, unsigned int value) ARGS_NONNULL;


/**
 * @brief Set object parameter to new value
 * @param[in] obj Pointer to object
 * @param[in] parametername Name of parameter
 * @param[in] value New value as unsigned long
 *
 * @return 0 or TR069 error code
 */
unsigned int dmos_setObjectParameterUnsignedLong (struct dm_object *obj, char *parametername, unsigned long value) ARGS_NONNULL;


/**
 * @brief Set object parameter to new value and check if notification is necessary
 *
 * @param[in] obj  Pointer to object
 * @param[in] parametername Name of parameter
 * @param[in] value New value as char pointer
 *
 * @return 0 or TR069 error code
 */
unsigned int dmos_setObjectParameterWithNotify (struct dm_object *obj, char *parametername, char *value) ARGS_NONNULL;


/**
 * @brief Set multiple object parameters to new values
 *
 * @param[in] obj  Pointer to object
 * @param[in] keyvaluepairs An array of char pointers, odd ones pointing to the
 *            parameter name, even ones pointing to the value; terminate
 *            with NULL
 *
 * @return 0 or TR069 error code
 */
unsigned int dmos_setMultipleObjectParameter (struct dm_object *obj, char *keyvaluepairs[]) ARGS_NONNULL;


/**
 * @brief update object
 *
 * @param[in] obj object to be updated
 *
 * @return 0 or error code
 *
 * @details Calls all defined parameter read function to update
 */
unsigned int dmos_updateObject (struct dm_object *obj) ARGS_NONNULL;


/**
 * @brief update object tree
 * @param[in] obj root object to be updated
 *
 * @return 0 or error code
 *
 * Calls all defined parameter read function from the object and all its sub objects
 */
unsigned int dmos_updateObjectTree (struct dm_object *obj) ARGS_NONNULL;


/**
 * @brief Set parameter indicated in path to new value while preventing notifications
 *
 * @param[in] root	root object of the datamodel
 * @param[in] pathname	parameter path
 * @param[in] value	new value as char pointer
 *
 * @return 0 for success or error code
 *
 * @details This functions does not evaluate the notification settings and also
 *           resets the internal change marks to prevent future notification on this change.
 *
 *           To be used for settings during init.
 */
unsigned int dmos_setPathParameterNoNotify (struct dm_object *root, char *pathname, char *value) ARGS_NONNULL;


/**
 * @brief Set parameter indicated in path to new value and check if notification is necessary
 *
 * @param[in] root	root object of the datamodel
 * @param[in] pathname  parameter path
 * @param[in] value	new value as char pointer
 *
 * @return 0 for success or error code
 */
unsigned int dmos_setPathParameterWithNotify (struct dm_object *root, char *pathname, char *value) ARGS_NONNULL;


/**
 * @brief indicate end of set operations
 *
 * @param[in] root	root object of the datamodel
 *
 * @details function is used to trigger a check if notification are pending and a session has to initiated
 */
void dmos_setObjectsDone (struct dm_object *root) ARGS_NONNULL DEPRECATED("No need to instruct the DM to check for changes anymore");


/**@}*/


/* == OBJECT PROPERTIES ROUTINES == */
/** @defgroup dmos_prop DMOS Object properties access routines
 *
 * These functions provide access to the object properties.
 *
 * @warning Do not try to access these properties directly over
 * the object parameters, as some of these properties are retrieved dynamically, and the object parameters values may be invalid.
 * @{
 */


 /**
 * @brief get internal object arguments
 * @param[in] obj  Pointer to object
 *
 * @return arguments string or NULL if no arguments are set
 *
 * @details This function may be used to retrieve the internal object arguments which are stored
 *          with \ref dmos_setObjectArgs. These arguments are normally used to can be used to
 *          identify a system resource, e.g. an Ethernet interface.
 */
char *dmos_getObjectArgs (struct dm_object *obj) ARGS_NONNULL RETURNS_NONNULL;

/**
 * @brief set internal object arguments
 * @param[in] obj  Pointer to object
 * @param[in] args string with arguments
 *
 * @return 0 if successful, or error code
 *
 * @details This function may be used to store additional arguments which are not TR-069 related
 *          with an object (instance) These arguments are normally used to connect the TR-069
 *          object (instance) with a system resource, e.g. to know that the object
 *          "Device.IP.Interface.4." points to eth3.
 *          To retrieve the data use \ref dmos_getObjectArgs.
 */
unsigned int dmos_setObjectArgs (struct dm_object *obj, char *args) ARGS_NONNULL;


 /**
 * @brief get object name
 * @param[in] obj  Pointer to object
 *
 * @return pointer to the name
 */
char *dmos_getObjectName (struct dm_object *obj) ARGS_NONNULL RETURNS_NONNULL;


/**@}*/


/* == PARAMETER PROPERTIES ROUTINES == */
 /** @defgroup dmos_pprop DMOS Parameter properties access routines
  *
  * These functions provide access to the parameter properties, if a parameter pointer is available
  *
  * @warning Do not try to access these properties directly over
  * the object parameters, as some of these properties are retrieved dynamically, and the object parameters values may be invalid.
  * @{
  */

 /**
 * @brief get name of the parameter
 * @param[in] parameter  Pointer to parameter
 *
 * @return Name of the parameter
 */
 char *dmos_getParameterName (struct dm_parameter *parameter) ARGS_NONNULL;

 /**
 * @brief get value of the parameter
 * @param[in] parameter  Pointer to parameter
 *
 * @return Value of the parameter
 */
 char *dmos_getParameterValue (struct dm_parameter *parameter) ARGS_NONNULL RETURNS_NONNULL;


 /**
 * @brief compares the actual parameter value with the new value for identity
 *
 * @param[in] parameter pointer to parameter
 * @param[in] newvalue new value for comparison
 *
 * @return 1 if values are different, 0 otherwise
 */
unsigned int dmos_isChangedParameterValue (struct dm_parameter *parameter, char *newvalue) ARGS_NONNULL;


 /**
 * @brief check if given parameter is the one specified.
 * @param[in] parameter  Pointer to a parameter
 * @param[in] parametername Name of parameter
 *
 * @return 1 if parameter is the one parameter specified, otherwise 0
 *
 * @details The function compares the given parameter name with the name of the specified parameter
 *          and returns 1 if these are equal, 0 in all other cases.
 */
unsigned int dmos_isParameter (struct dm_parameter *parameter, char *parametername) ARGS_NONNULL;

/**@}*/


/* == Table Handling ROUTINES == */
/** @defgroup dmos_table DMOS Table handling routines
 * The table handling allow to work with table objects. These routines will return errors if called
 * pointing to a static object.
 *
 * \note These functions do not check if a table is writable or not because they are used
 * internally. For tables which are managed by the ACS, no instances should be created or deleted
 * in the mapping functions
 * @{
 */

 /**
 * @brief get next table instance
 * @param[in] prev_instance  pointer either to Table root to retrieve first instance or to an existing instance
 * @param[in] create  create a new instance if there is no one remaining
 *
 * @return pointer to new instance or NULL if no one was found or object is not a table
 *
 */
 struct dm_object *dmos_getNextTableInstance (struct dm_object *prev_instance, unsigned int create) ARGS_NONNULL;


 /**
 * @brief add new table instance
 * @param[in] table pointer either to table root or to an existing instance
 * @param[in,out] instance reference to variable for instance number
 *
 * @return pointer to new instance or NULL if no one was found or object is not a table
 */
 struct dm_object *dmos_addNewTableInstance (struct dm_object *table, unsigned int *instance) ARGS_NONNULL;


 /**
 * @brief get number of instances in a table
 * @param[in] obj pointer either to Table root to an table instance
 *
 * @return Number of instances or -1 if object is not a table or table instance
 */
int dmos_getNumberOfTableInstances (struct dm_object *obj) ARGS_NONNULL;


 /**
 * @brief rename/renumber a table instance e.g. to match indices on the system
 * @param[in] instance pointer to a table instance
 * @param[in] instno new instance number to assign
 *
 * @return 0 if successful or TR-069 error code otherwise
 */
unsigned int dmos_renameTableInstance (struct dm_object *instance, unsigned int instno) ARGS_NONNULL;


 /**
 * @brief Delete all remaining table instances
 *
 * @param[in] prev_instance pointer either to table root to remove all
 *            instances or to an existing instance to remove all following instances
 *
 * @return 0 if the instances could be deleted or there are none to delete, or an error code
 *
 */
unsigned int dmos_deleteNextTableInstances (struct dm_object **prev_instance) ARGS_NONNULL;


/**
 * @brief find instance in table with given args
 * @param[in] table  object table pointer
 * @param[in] args string with arguments
 *
 * @return pointer to instance if successful, or NULL
 */
struct dm_object *dmos_findTableInstanceWithArgs (struct dm_object *table, char *args) ARGS_NONNULL;


/**
 * @brief delete given object instance
 * @param[in] obj pointer to table instance
 *
 * @return 0 if success or TR_069 error code
 */
unsigned int dmos_deleteObjectInstance (struct dm_object *obj) ARGS_NONNULL;

/**@}*/





/* == Miscellaneous ROUTINES == */
/** @defgroup dmos_misc DMOS miscellaneous routines
 * These are support routines to be used for DM operations.
 * @{
 */

/**
* @brief convert boolean char value into int
* @param[in] value boolean char value from data model
*
* @return int {0,1,-1} 0: false, 1: true, -1 non boolean value
*/
int dmos_convertChar2Boolean (char *value) ARGS_NONNULL;


/**
* @brief convert a SOAP DateTime string to a Unix time stamp
* @param[in] value a string describing a TR-069/SOAP DateTime
*
* @return time_t a Unix time stamp
*
*/
time_t dmos_convertDateTimeToUnixtime (char *value);


/**
* @brief convert a Unix time stamp to a SOAP DateTime string
* @param[in] tm a Unix time stamp
*
* @return A freshly allocated string describing a TR-069/SOAP DateTime
*/
char *dmos_convertUnixtimeToDateTime (time_t tm);

/**@}*/


/* =========== Out of session functions ========= */

/** @defgroup osdm_out OSDM Out of session functions
 * The OSDM interface provides additional functions of the Mapping Support Library, which may only be called
 * out of TR-069 session processing. They can be used if an update of the data model is requested by
 * local tasks, like proxy clients, local processes.
 *
 * \note <b>These functions MUST never be used within the Data model mapping functions, which are called during RPC
 * processing !!!</b>
 *
 * These function will trigger new sessions if necessary.
 *
 * Usage examples are:
 * - Trigger a value change after a new table had been added
 * - Save new values, which are updated i the DM out of session
 * @{
 */


/**
 * @brief Check for pending active notification and handle them
 *
 * @param[in] obj hints the changed object, if NULL will search all parameters
 * 	      with active notification enabled
 *
 * @details This function will check whether a session is ongoing and if not
 * 	    update the specified object (or all objects with active
 * 	    notification enabled if NULL) and in case of changes open a new
 * 	    TR-069 session sending handling those changes accordingly. If a
 * 	    session is ongoing the value change check will be scheduled until
 * 	    after the ongoing session.
 */
void osdm_triggerActiveNotificationCheck (struct dm_object *obj) DEPRECATED("No need to instruct the DM to check for changes anymore");


/**
* @brief add new table instance and check for notification
* @param[in] table  pointer either to Table root  or to an existing instance
* @param[out] instance reference to variable for instance number
* @return pointer to new instance or NULL if no one was found or object is not a table
*
* @details This function works as the \ref dmos_addNewTableInstance function, but additionally a
*          Value change event is triggered, if an active notification has been set on the table counter.
*/
struct dm_object *osdm_addTableInstanceWithNotify (struct dm_object *table, unsigned int *instance) ARGS_NONNULL;


/**
* @brief delete table instance and check for notification
* @param[in] instance  pointer to an existing instance
*
* @return 0 for success or TR-069 error code
*
* @details This function first deletes the instance and if necessary triggers
*          Value change event, if a notification has been set on the table counter.
*/
unsigned int osdm_delTableInstanceWithNotify (struct dm_object *instance) ARGS_NONNULL;


/**
 * @brief trigger the check for pending notifications
 *
 * @param[in] obj pointer to object to be notified or NULL to check all objects for active notification
 */
void osdm_trigger_active_notification (struct dm_object *obj) DEPRECATED("No need to instruct the DM to check for changes anymore");


/**@}*/
#endif /* dmos_H_ */
